/*
 * Java help functions. Displays function protypes for common java classes.
 * Author Jim Morris, splash@wolfman.com
 *
 * To use just put the java*.tags files in the epsilon folder, and the
 * javahelp.e in the source folder, load it and ALT-F3 to compile and load and
 * CTRL-F3 to save.
 * 
 * ALT-F1 will extract the java method name to the left of the
 * current cursor position, then search for that function in the
 * current javahelp file, displaying the results in a popup window (or
 * in the message line if there is only one match). All
 * classes that have that public method are displayed.
 * ALT-F2 will lookup up the selected class name or prompt for a class
 * name, and show all the public methods for that class.
 * In both cases the results show the function prototype and
 * parameters.
 * There are currently two java help files, java118.tags and
 * java131.tags, the default is set to java118.tags and has the most
 * commonly used public methods for all java version 1.1.8 functions,
 * java131.tags has the java2 version 1.3.1 functions.
 * The variable javahelpfile can be set to which ever version is
 * needed.
 * 
 * The help files are generated by running a program called jtags with
 * the following options... jtags -i =f -L input.txt where input.txt
 * has...
 * $JDKSRC\jdk1.1.8\src\java\io
 * $JDKSRC\jdk1.1.8\src\java\lang
 * $JDKSRC\jdk1.1.8\src\java\math
 * $JDKSRC\jdk1.1.8\src\java\net
 * $JDKSRC\jdk1.1.8\src\java\text
 * $JDKSRC\jdk1.1.8\src\java\util
 * Where $JDKSRC is the java source distribution. You could add swing
 * etc if you wanted.
 * Jtags can be found at: http://www.fleiner.com/jtags/
 * and I have added a few patches to show parameters split over multiple
 * lines, just replace the file TagVector.java with my version and recompile.
 */

#include "eel.h"

char javahelpfile[]= "java118.tags";
//char javahelpfile[]= "java131.tags";
char lastjavahelpfile[256]= "";

int open_java_help_file()
{
    int i;
    char *path;
    if(strcmp(lastjavahelpfile, javahelpfile) != 0){ // changed since last run
	delete_buffer("-javahelp");
    }
    
    if(!exist("-javahelp")){
	create("-javahelp");
	bufname= "-javahelp";
	path= lookpath(javahelpfile);
	if ( !path ) {
	    delete_buffer("-javahelp");
	    error ( "Can't find file %s", javahelpfile );
	}
	i = err_file_read(path, FILETYPE_AUTO);
	if (i) {			/* error reading file */
	    delete_buffer("-javahelp");
	    error ( "Can't open file %s", path);
	}
    }else{
	bufname= "-javahelp";
	point= 0;
    }
    strcpy(lastjavahelpfile, javahelpfile);
    return 1;
}

/* read a function name at point & find it in the help file, then display the function prototype */
command get_parameter_help() on cx_tab['?'], reg_tab[NUMALT(FKEY(1))]
{
    char tag[TAGLEN], *tag_pat;
    int helpbufnum;
    int i;
    char *original_buffer;
    char pat[132], s1[132], s2[132], s3[132];
    char helpstr[512];
    int p1, p2, p3, p4, p5;
    int oldpos= point;
    int cnt= 0;
    char *np;
    char wintitle[256];
    
    // find the function name to get help on
    tag_pat = get_mode_string_variable("tag-pattern-%s");
    if (got_bad_number || !tag_pat)
	return;
    tiled_only();
    iter = 0;
    re_search(-1, tag_pat);
    grab(point, matchstart, tag);

    // get the javahelp file and search it
    original_buffer = bufname;
    helpbufnum= zap("-paramhelp"); // buffer to put results in

    open_java_help_file();
  
    // find each occurrence, parse out class and call and copy it to the help buffer
    sprintf(pat, "^(%s)<Tab>(.*)<Tab>/<Caret><Space|Tab>+(<^Space|Tab>.*)%%$/;", tag);
    while(re_search(1, pat)){
	p1= find_group(1, 1);
	p2= find_group(2, 1);
	p3= find_group(2, 0);
	p4= find_group(3, 1);
	p5= find_group(3, 0);
	
	grab(p1, p2-1, s1);	// the function name
	grab(p2, p3-5, s2);	// the full path to the .java file
	grab(p4, p5-1, s3);	// the function prototype

	np= rindex(s2, '\\'); // extract filename and strip the .java
	sprintf(helpstr, "%s - %s", np+1, s3);
	buf_printf(helpbufnum, "%s\n", helpstr);
	cnt++;
    }

    // now display the results
    bufname= original_buffer;
    point = oldpos;
    
    if(cnt == 0){
	say("%s Not found", tag);
	delete_buffer("-paramhelp");
	quick_abort();
    }else if(cnt == 1){
	say(helpstr);
    }else{
	sort_another("-paramhelp", 0, 0);
	sprintf(wintitle, "Matches for method: %s", tag);
	save_var _view_title = wintitle;
	view_buf(helpbufnum, 0);
	delete_buffer("-paramhelp");    
    }
}

/*
read(

readByte(   

readLine(

arraycopy(

entrySet(
*/

/* prompt for a class name & find it in the help file, then display
 * all the function prototypes for that class
 */
command get_class_functions_help() on reg_tab[NUMALT(FKEY(2))]
{
    char tag[TAGLEN];
    int helpbufnum;
    int i;
    char *original_buffer;
    char pat[132], s1[132], s2[132], s3[132];
    char helpstr[512];
    int p1, p2, p3, p4, p5;
    int oldpos= point;
    int cnt= 0;
    char *np;
    char wintitle[256];
    
    save_spot point, mark;
    
    fix_region();

    // find the function name to get help on, either use selected
    // region or prompt for it
    if (is_highlight_on() && mark - point < sizeof(tag)){
	grab(point, mark, tag);
	highlight_off();
    }else{
	get_string(tag, "function for class: ");
    }
    
    // get the javahelp file and search it
    original_buffer = bufname;
    helpbufnum= zap("-paramhelp"); // buffer to put results in

    open_java_help_file();

    // find each occurrence, parse out class and call and copy it to the help buffer
    sprintf(pat, "^(.*)<Tab>(.*\\%s%%.java)<Tab>/<Caret><Space|Tab>+(<^Space|Tab>.*)%%$/;", tag);
    while(re_search(1, pat)){
	p1= find_group(1, 1);
	p2= find_group(2, 1);
	p3= find_group(2, 0);
	p4= find_group(3, 1);
	p5= find_group(3, 0);

	grab(p1, p2-1, s1);	// the function name
	grab(p2, p3-5, s2);	// the full path to the .java file
	grab(p4, p5-1, s3);	// the function prototype

	if(cnt == 0){ // show class name
	    sprintf(wintitle, "Class: %s", s2+16);
	}
	    
	sprintf(helpstr, "%s", s3);
	buf_printf(helpbufnum, "%s\n", helpstr);
	cnt++;
    }

    // now display the results
    bufname= original_buffer;
    point = oldpos;

    if(cnt == 0){
	say("%s Not found", tag);
	delete_buffer("-paramhelp");
	quick_abort();
    }else{
	save_var _view_title = wintitle;
	view_buf(helpbufnum, 0);
	delete_buffer("-paramhelp");    
    }
}
